// MyToday1.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "MyToday1.h"

LPCTSTR g_lpcszClass = _T("MyToday1WndClass");
LPCTSTR g_lpcszWnd = _T("MyToday1Wnd");
HWND g_mainWnd = NULL;
HINSTANCE g_hInstance = NULL;

// Icons to draw
HICON g_hAppIcon = NULL;
HICON g_hPowerIcon = NULL;

// Flags for drawings
BOOL g_bShowFirst = TRUE;
BOOL g_bShowSecond = TRUE;
BOOL g_bShowPower = TRUE;

// Default settings for today fonts and texts
COLORREF g_crTodayText = NULL;
HFONT g_fontTodayNormal = NULL;
HFONT g_fontTodayBold = NULL;

BOOL APIENTRY DllMain(HANDLE hModule, DWORD dwReason, LPVOID lpReserved)
{
	if (dwReason == DLL_PROCESS_ATTACH)
	{
		// Just remember DLL instance
		g_hInstance = (HINSTANCE)hModule;

		// Unregister today window class
		UnregisterClass(g_lpcszClass, (HINSTANCE)hModule);

		// Register today window class
		WNDCLASS wndClass;
		memset(&wndClass, 0, sizeof(wndClass));
		wndClass.hCursor = 0;
		wndClass.hIcon = 0;
		wndClass.hInstance = (HINSTANCE)hModule;
		wndClass.lpszClassName = g_lpcszClass;
		wndClass.lpszMenuName = NULL;
		wndClass.style = CS_VREDRAW | CS_HREDRAW;
		wndClass.hbrBackground = (HBRUSH)GetStockObject(HOLLOW_BRUSH);
		wndClass.lpfnWndProc = WndProc;

		RegisterClass(&wndClass);
	}
	else if (dwReason == DLL_PROCESS_DETACH)
	{
		// Destroy all allocated variables
		if (g_fontTodayNormal)
			DeleteObject(g_fontTodayNormal);

		if (g_fontTodayNormal)
			DeleteObject(g_fontTodayBold);

		if (g_hAppIcon)
			DeleteObject(g_hAppIcon);

		if (g_hPowerIcon)
			DeleteObject(g_hPowerIcon);
	}

    return TRUE;
}

void GetTodayDefaults(HWND hWndParent)
{
	g_crTodayText = SendMessage(hWndParent, TODAYM_GETCOLOR, (WPARAM)TODAYCOLOR_TEXT, NULL);

	LOGFONT lf;
	HFONT hSysFont;

	// Default today fonts
	hSysFont = (HFONT)GetStockObject(SYSTEM_FONT);
	GetObject(hSysFont, sizeof(LOGFONT), &lf);

	HDC hdc = GetDC(g_mainWnd);

	lstrcpy(lf.lfFaceName, _T("Tahoma"));
	lf.lfHeight = -1 * (17 * GetDeviceCaps(hdc, LOGPIXELSY) / 72) / 2;

	ReleaseDC(g_mainWnd, hdc);

	// Normal text
	lf.lfWeight = FW_NORMAL;
	g_fontTodayNormal = CreateFontIndirect(&lf);

	// Bold text
	lf.lfWeight = FW_BOLD;
	g_fontTodayBold = CreateFontIndirect(&lf);
}

void LoadIcons()
{
	// Load app icon which will be drawn on the left of today item
	g_hAppIcon = (HICON)LoadImage(g_hInstance, MAKEINTRESOURCE(IDI_APP_ICON), IMAGE_ICON, 16, 16, LR_DEFAULTCOLOR);
	// Load additional icon used just for painting of today item
	g_hPowerIcon = LoadIcon(g_hInstance, MAKEINTRESOURCE(IDI_POWER_ICON));
}

HWND APIENTRY InitializeCustomItem(TODAYLISTITEM *ptli, HWND hWndParent)
{
	// Check if item's enabled
	if (!ptli->fEnabled)
		return 0;

	// Set zero initial height
	ptli->cyp = 0;

	// Create window itself, with desired style
	// Again set its zero height
	g_mainWnd = CreateWindow(g_lpcszClass, g_lpcszWnd, WS_CHILD | WS_VISIBLE,
		                     // Set correct size
		                     0, 0, GetSystemMetrics(SM_CXSCREEN), 0,
							 hWndParent, NULL, ptli->hinstDLL, NULL);

	if (!g_mainWnd)
		return 0;

	// Get defaults for today screen
	GetTodayDefaults(hWndParent);

	// Load all icons needed
	LoadIcons();

	return g_mainWnd;
}

void DoPaint(HDC hDC)
{
	HFONT hFontOld;
	RECT rect;
	SIZE size;
	LPCTSTR szText;
	TCHAR *lpszText = (TCHAR*)malloc(MAX_PATH);

	// Set transparent background
	SetBkMode(hDC, TRANSPARENT);
	// Set font color to Today text one
	SetTextColor(hDC, g_crTodayText);

	// Draw Today item icon
	if (g_hAppIcon)
		DrawIcon(hDC, 2, 0, g_hAppIcon);

	// First draw text using bold today font
	if (g_bShowFirst)
	{
		szText = _T("First line - bold font");

		hFontOld = (HFONT)SelectObject(hDC, g_fontTodayBold);

		GetTextExtentPoint32(hDC, szText, _tcslen(szText), &size);
		rect.left = 30;
		rect.top = 0;
		rect.right = rect.left + size.cx;
		rect.bottom = size.cy;

		DrawText(hDC, szText, -1, &rect, DT_LEFT);

		SelectObject(hDC, hFontOld);
	}

	// Then draw text using normal today font
	if (g_bShowSecond)
	{
		szText = _T("Second line - normal font");
		hFontOld = (HFONT)SelectObject(hDC, g_fontTodayNormal);

		GetTextExtentPoint32(hDC, szText, _tcslen(szText), &size);
		rect.left = 30;
		rect.top = 15;
		rect.right = rect.left + size.cx;
		rect.bottom = rect.top + size.cy;

		DrawText(hDC, szText, -1, &rect, DT_LEFT);

		SelectObject(hDC, hFontOld);
	}

	if (g_hPowerIcon && g_bShowPower)
		DrawIcon(hDC, 200, 2, g_hPowerIcon);
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_PAINT :
		{
            PAINTSTRUCT ps;
            HDC hDC;

            hDC = BeginPaint(hWnd, &ps);

			// Draw Today Item
			DoPaint(hDC);

			EndPaint(hWnd, &ps);
		}

		break;
	case WM_ERASEBKGND :
		{
			HDC hDC = (HDC)wParam;
			TODAYDRAWWATERMARKINFO dwi;

			dwi.hdc = hDC;
			GetClientRect(hWnd, &dwi.rc);		
			dwi.hwnd = hWnd;

			SendMessage(GetParent(hWnd), TODAYM_DRAWWATERMARK, 0, (LPARAM)&dwi);

			return TRUE;
		}
	case WM_TODAYCUSTOM_CLEARCACHE :
		{
			TODAYLISTITEM *ptli = (TODAYLISTITEM *)wParam;

			return FALSE;
		}
	case WM_TODAYCUSTOM_QUERYREFRESHCACHE :
		{
			TODAYLISTITEM *ptli = (TODAYLISTITEM *)wParam;

			if (ptli->cyp == 0)
			{
				ptli->cyp = 40;
				return TRUE;
			}

			return FALSE;
		}
	case WM_LBUTTONUP :
		{
			POINT point;
			point.x = LOWORD(lParam);
			point.y = HIWORD(lParam);

			if (g_bShowPower)
			{
				RECT rect;
				rect.left = 200;
				rect.top = 2;
				rect.right = rect.left + 32;
				rect.bottom = rect.top + 32;

				if (PtInRect(&rect, point))
				{
					// Click on power icon runs Power control panel applet
					PROCESS_INFORMATION pi;

					::CreateProcess(_T("\\Windows\\ctlpnl.exe"), _T("cplmain.cpl,3"), NULL, NULL, FALSE, 0, NULL, NULL, NULL, &pi);
				}
			}

			return 0;
		}
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

// Option Dialog
HFONT g_fontTitle = NULL;
SHACTIVATEINFO g_sai;
LPCTSTR g_lpcszOptions = _T("My Today Options");
int g_nTitleHeight = 23;

// Support routine for drawing proper dialog title
void GetDefaultTitleFont(HWND hWnd)
{
	LOGFONT lf;
	TEXTMETRIC tm;
	HDC hdc;
	
	hdc = GetDC(hWnd);
	GetTextMetrics(hdc, &tm);
	memset(&lf, 0, sizeof (lf));
	lf.lfHeight = -1 * (17 * GetDeviceCaps(hdc, LOGPIXELSY) / 72) / 2;
	lf.lfWeight = FW_SEMIBOLD;
	lf.lfPitchAndFamily = tm.tmPitchAndFamily;
	lstrcpy(lf.lfFaceName, _T("Tahoma"));
	g_fontTitle = CreateFontIndirect(&lf);
	ReleaseDC(hWnd, hdc);
}

void DrawDialogTitle(HWND hWnd, HDC hDC)
{
	if (lstrlen(g_lpcszOptions))
	{
		RECT rect, rectText;
		GetClientRect(hWnd, &rect);

		// Draw text
		HFONT hOldFont = (HFONT)SelectObject(hDC, g_fontTitle);
		SetTextColor(hDC, RGB(0, 0, 156));

		rectText.left = 8;
		rectText.top = 0;
		rectText.right = rect.right - rect.left;
		rectText.bottom = g_nTitleHeight;
		DrawText(hDC, g_lpcszOptions, -1, &rectText, DT_VCENTER);
		SelectObject(hDC, hOldFont);

		// And finally underline title
		HPEN hOldPen, hPen = (HPEN)GetStockObject(BLACK_PEN);
		rect.top = rect.top + g_nTitleHeight;
		rect.bottom = rect.top;

		hOldPen = (HPEN)SelectObject(hDC, hPen);
		Polyline(hDC, (LPPOINT)&rect, 2);
		SelectObject(hDC, hOldPen);
	}
}

LRESULT WINAPI CustomItemOptionsDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG :
		{
			// Initialize structure for WM_SETTINGCHANGE and WM_ACTIVATE
			memset(&g_sai, 0, sizeof(SHACTIVATEINFO));
			g_sai.cbSize = sizeof(SHACTIVATEINFO);

			// lParam contains pointer to Today Item Structure
			TODAYLISTITEM *ptli = (TODAYLISTITEM*)lParam;

			// We need to do correct initialization of our dialog
			SHINITDLGINFO shidi;
			shidi.dwMask = SHIDIM_FLAGS;
			shidi.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN;
			shidi.dwFlags |= SHIDIF_SIZEDLGFULLSCREEN;
			shidi.hDlg = hWnd;

			SHInitDialog(&shidi);

			// Now we can get default title font
			// Optional - just for better look as recommended by Microsoft
			GetDefaultTitleFont(hWnd);

			// Initialize check boxes
			SendMessage(GetDlgItem(hWnd, IDC_CHECK_FIRST), BM_SETCHECK, g_bShowFirst ? BST_CHECKED : BST_UNCHECKED, 0);
			SendMessage(GetDlgItem(hWnd, IDC_CHECK_SECOND), BM_SETCHECK, g_bShowSecond ? BST_CHECKED : BST_UNCHECKED, 0);
			SendMessage(GetDlgItem(hWnd, IDC_CHECK_POWER), BM_SETCHECK, g_bShowPower ? BST_CHECKED : BST_UNCHECKED, 0);

			return TRUE;
		}
	case WM_SETTINGCHANGE :
		{
			SHHandleWMSettingChange(hWnd, wParam, lParam, &g_sai);
		}

		return 0;
	case WM_ACTIVATE :
		{
			SHHandleWMActivate(hWnd, wParam, lParam, &g_sai, FALSE);
		}

		return 0;
	case WM_PAINT :
		{
            PAINTSTRUCT ps;
            HDC hDC;

            hDC = BeginPaint(hWnd, &ps);

			// Draw title bar for our dialog
			DrawDialogTitle(hWnd, hDC);

			EndPaint(hWnd, &ps);
		}

		break;
	case WM_COMMAND :
		{
			UINT nID = LOWORD(wParam);

			// Handle closing of dialog
			if (nID == IDOK)
			{
				// Destroy title font
				if (g_fontTitle)
					DeleteObject(g_fontTitle);

				// Set variables for Today Item
				g_bShowFirst = (SendMessage(GetDlgItem(hWnd, IDC_CHECK_FIRST), BM_GETCHECK, 0, 0) == BST_CHECKED);
				g_bShowSecond = (SendMessage(GetDlgItem(hWnd, IDC_CHECK_SECOND), BM_GETCHECK, 0, 0) == BST_CHECKED);
				g_bShowPower = (SendMessage(GetDlgItem(hWnd, IDC_CHECK_POWER), BM_GETCHECK, 0, 0) == BST_CHECKED);

				EndDialog(hWnd, IDOK);
				return 0;
			}
		}

		break;
	}

	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}
